/*
 * Copyright (c) 2017 Texas Instruments Incorporated - http://www.ti.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== ReentSupport.c ========
 */

#include <xdc/std.h>
#include <xdc/runtime/Assert.h>
#include <ti/sysbios/BIOS.h>

#include "package/internal/ReentSupport.xdc.h"

static int errno; /* used only while in main thread */

/*
 *************************************************************************
 *                       Internal functions
 *************************************************************************
 */

/*
 *  ======== ReentSupport_taskRegHook ========
 *  Store assigned hookset ID
 *
 *  The Task register hook is called once per hookset, before main and
 *  before any Task initialization has been done.
 *
 *  This function allows the ReentSupport module to store its assigned
 *  hookset ID, which is needed when calling Task_getHookContext and
 *  Task_setHookContext. The HookContext can be an arbitrary structure.
 *  The ReentSupport module has defined a HookContext structure.
 */
void ReentSupport_taskRegHook(Int id)
{
    ReentSupport_module->taskHId = id;
}

/*
 *  ======== ReentSupport_getReent ========
 */
Int *ReentSupport_getReent()
{
    Int *buf;

    Assert_isTrue(((BIOS_getThreadType() != BIOS_ThreadType_Hwi)
            && (BIOS_getThreadType() != BIOS_ThreadType_Swi)),
            ReentSupport_A_badThreadType);

    /* if still in the main thread, use local errno above */
    if (BIOS_getThreadType() == BIOS_ThreadType_Main) {
        buf = (Int *)(&errno);
    }
    else {
        /*  Invoke internal function (generated by template), to get
         *  address of TLS. This function implementation will differ
         *  depending if SYS/BIOS is in ROM or not.
         */
        buf = ReentSupport_getTlsAddr();
    }

    return (buf);
}
